# Copyright (c) 2005, University of Massachusetts
# All Rights Reserved.
#
# Name:           generate_par.prom
# Last modified:  30 Mar 2005
# Author:         Stephen Siegel <siegel.umass.edu>
#
# This perl script will generate the parallel models for the
# Monte Carlo pi estimation program from Using MPI. Usage:
# perl generate_par.pl.
#

# Global variables...

$filename = "";
$nprocs = 0;


sub generate_mpi {
  my($i);

  foreach $i (1..$nprocs-1) {
    print OUT "chan MPI_Bcast_chan_$i = [BCAST_CHAN_SIZE] of {Expr};\n";
  }
  print OUT "\n";
  foreach $i (0..$nprocs-2) {
    print OUT "chan MPI_server_in_chan_$i = [SERVER_CHAN_SIZE] of {bit};\n";
  }
  print OUT "\n";
  foreach $i (0..$nprocs-2) {
    print OUT "chan MPI_server_out_chan_$i = [SERVER_CHAN_SIZE] of {Array};\n";
  }
  print OUT "\n";
  foreach $i (0..$nprocs-2) {
    print OUT "chan MPI_coord_in_chan_$i = [COORD_CHAN_SIZE] of {Expr};\n";
  }
  print OUT "\n";
  foreach $i (0..$nprocs-2) {
    print OUT "chan MPI_coord_out_chan_$i = [COORD_CHAN_SIZE] of {Expr};\n";
  }
  print OUT "\n";
  print OUT "inline monte_par() {
#if DEBUG>0
  c_code{ printf(\"Starting MPI procs...\\n\"); };
#endif
";
  print OUT "  run MPI_Coordinator();\n";
  foreach $i (0..$nprocs-2) {
    print OUT "  run Worker_$i();\n";
  }
  print OUT "  run Server();
#if DEBUG>0
  c_code{ printf(\"MPI procs started.\\n\"); };
#endif
";
  print OUT "}\n";

  print OUT "
proctype MPI_Coordinator() {
  Expr datum = SYM_ZERO;
  Expr total = SYM_ZERO;\n\n";

  print OUT "#if COORD_CHAN_SIZE>0\n";
  foreach $i (0..$nprocs-2) {
    print OUT "  xr MPI_coord_in_chan_$i;\n";
    print OUT "  xs MPI_coord_out_chan_$i;\n";
  }
  print OUT "#endif\n";

  print OUT "
  atomic {
end_c:
    do
    :: MPI_coord_in_chan_0?total;
#if DEBUG_COORD
       c_code{
         printf(\"Coordinator received e%d\\n\", PMPI_Coordinator->total);
       };
#endif
";
  foreach $i (1..$nprocs-2) {
    print OUT "       MPI_coord_in_chan_$i?datum;\n";

    print OUT "
       c_code {
#if DEBUG_COORD
         printf(\"Coordinator received e%d\\n\", PMPI_Coordinator->datum);
#endif
";
    print OUT "         PMPI_Coordinator->total =\n ";
    print OUT "           SYM_add_int(PMPI_Coordinator->total, PMPI_Coordinator->datum);\n";
    print OUT "         PMPI_Coordinator->datum = SYM_ZERO;\n";
    print OUT "       };\n";
  }
  print OUT "
#if DEBUG_COORD
       c_code{
         printf(\"Coordinator sending e%d\\n\", PMPI_Coordinator->total);
       };
#endif
";
  foreach $i (0..$nprocs-2) {
    print OUT "       MPI_coord_out_chan_$i!total;\n";
  }
  print OUT "    od\n";
  print OUT "  }\n";
  print OUT "}\n\n";
}


sub generate_worker {
  my($wid) = $_[0];
  my($i);
  my($pre) = "PWorker_$wid";

  print OUT "
proctype Worker_$wid() {
  bit request = 1;
  byte done = 0;
  Expr in = SYM_ZERO;
  Expr out = SYM_ZERO;
  Expr totalin = SYM_ZERO;
  Expr totalout = SYM_ZERO;
  Expr Pi, epsilon;
  Array rands;

  atomic {

#if DEBUG > 0
    c_code{ printf(\"Worker_$wid started.\\n\"); };
#endif

#if SERVER_CHAN_SIZE > 0
    xs MPI_server_in_chan_$wid;
    xr MPI_server_out_chan_$wid;
#endif

#if COORD_CHAN_SIZE > 0
    xs MPI_coord_in_chan_$wid;
    xr MPI_coord_out_chan_$wid;
#endif
";

  print OUT "\n#if BCAST_CHAN_SIZE > 0\n";
  if ($wid == 0) {
    foreach $i (1..$nprocs-1) {
      print OUT "    xs MPI_Bcast_chan_$i;\n";
    }
  }
  else {
    print OUT "    xr MPI_Bcast_chan_$wid;\n";
  }
  print OUT "#endif\n\n";

  if ($wid == 0) {
    print OUT "    epsilon = EPSILON;
#if DEBUG_COMM > 0
    c_code{ printf(\"Worker_$wid starting bcast.\\n\"); };
#endif
";
    foreach $i (1..$nprocs-1) {
      print OUT "    MPI_Bcast_chan_$i!epsilon;
#if DEBUG_COMM > 0
    c_code{ printf(\"Worker_$wid completed bcast to proc $i.\\n\"); };
#endif
";
    }
    print OUT "
#if DEBUG_COMM > 0
    c_code{ printf(\"Worker_$wid completed bcast.\\n\"); };
#endif
";
  }
  else {
    print OUT "    MPI_Bcast_chan_$wid?epsilon;\n";
  }
  print OUT "
    MPI_server_in_chan_$wid!request;
    do
    :: !done ->
#if DEBUG_COMM > 0
       c_code{ printf(\"Worker_$wid awaiting rands.\\n\"); };
#endif
       request = 1;
       MPI_server_out_chan_$wid?rands;
       c_code {
         {
           int i=0;
           int tmp1, tmp2, x, y;

           while (i < CHUNKSIZE-1) {
             tmp1 = SYM_divide($pre->rands.entryAt[i], INT_MAX);
             $pre->rands.entryAt[i] = SYM_ZERO;
             i++;
             tmp1 = SYM_multiply(tmp1, SYM_TWO);
             x = SYM_subtract(tmp1, SYM_ONE);
             tmp1 = SYM_divide($pre->rands.entryAt[i], INT_MAX);
             $pre->rands.entryAt[i] = SYM_ZERO;
             i++;
             tmp1 = SYM_multiply(tmp1, SYM_TWO);
             y = SYM_subtract(tmp1, SYM_ONE);
             tmp1 = SYM_multiply(x, x);
             tmp2 = SYM_multiply(y, y);
             tmp1 = SYM_add(tmp1, tmp2);
             tmp2 = SYM_intLessThan(tmp1, SYM_ONE);
             $pre->in = SYM_add_int($pre->in, tmp2);
             tmp2 = SYM_negate(tmp2);
             tmp2 = SYM_add_int(SYM_ONE, tmp2);
             $pre->out = SYM_add_int($pre->out, tmp2);
           }
         }
       };
       MPI_coord_in_chan_$wid!in;
       MPI_coord_out_chan_$wid?totalin;
       MPI_coord_in_chan_$wid!out;
       MPI_coord_out_chan_$wid?totalout;
       c_code {
         int tmp1, tmp2, result, inoutsum, count, error;

         tmp1 = SYM_multiply(SYM_FOUR, $pre->totalin);
         tmp2 = SYM_add_int($pre->totalin, $pre->totalout);
         $pre->Pi = SYM_divide(tmp1, tmp2);
         tmp1 = SYM_subtract($pre->Pi, LITERAL_PI);
         error = SYM_abs(tmp1);
         result = SYM_isLessThan(now.pc, error, $pre->epsilon);
         inoutsum = SYM_add_int($pre->totalin, $pre->totalout);
         if (result==1) {
           $pre->done = 1;
         }
         else {
           if (SYM_isInt(inoutsum)) {
             if (SYM_intVal(inoutsum)>POINTS_BOUND) {
               $pre->done = 1;
             }
             else {
               $pre->done = 0;
             }
           }
           else {
             $pre->done = 2;
           }
         }
       };
       if
       :: done==2 -> if :: 1 -> done = 1 :: 1 -> done = 0 fi
       :: else
       fi;
       request = (done -> 0 : 1);
";
  if ($wid == 0) {
    print OUT "       MPI_server_in_chan_0!request;\n";
  }
  else {
    print OUT "
       if
       :: request -> MPI_server_in_chan_$wid!request;
       :: else
       fi\n";
  }
  print OUT "
    :: else -> break
    od;
    if
    :: totalin != in_global ->
       c_code { report_par_error(1, $pre->totalin, now.in_global); };
       assert(0)
    :: else
    fi;
    if
    :: totalout != out_global ->
       c_code { report_par_error(2, $pre->totalout, now.out_global); };
       assert(0)
    :: else
    fi;
    if
    :: Pi != Pi_global ->
       c_code { report_par_error(3, $pre->Pi, now.Pi_global); };
       assert(0)
    :: else
    fi
  }
}

";
}


sub generate_server {
  print OUT "
proctype Server() {
  bit request;
  Expr epsilon;
  Array rands;
#if SYMMETRY
  byte stage = 0;
#if NWORKERS>1
  bit served[NWORKERS] = 0;
#else
  bit served[2] = 0;
#endif
  byte nserved = 0;
#else
  byte randcount2 = 0;
#endif

#if BCAST_CHAN_SIZE > 0
  xr MPI_Bcast_chan_".($nprocs-1).";
#endif

#if SERVER_CHAN_SIZE > 0
";
  foreach $i (0..$nprocs-2) {
    print OUT "  xs MPI_server_out_chan_$i;\n";
    print OUT "  xr MPI_server_in_chan_$i;\n";
  }
  print OUT "
#endif

  MPI_Bcast_chan_".($nprocs-1)."?epsilon;
  do
  :: atomic {
       if\n";
  foreach $i (0..$nprocs-2) {
    print OUT "
       :: MPI_server_in_chan_$i?request ->
          if
          :: request ->
#if SYMMETRY
             assert(stage < MAXPOINTS/NWORKERS);
             assert(!served[$i]);
#endif
             c_code {
               int i;
#if SYMMETRY
               for (i=0; i<CHUNKSIZE; i++) {
                 PServer->rands.entryAt[i] =
                   RAND(PServer->stage*CHUNKSIZE_SEQ+$i*CHUNKSIZE+i);
               }
               PServer->served[$i] = 1;
               PServer->nserved++;
               if (PServer->nserved == NWORKERS) {
                 int j;

                 PServer->nserved = 0;
                 PServer->stage++;
                 for (j=0; j<NWORKERS; j++) {
                   PServer->served[j] = 0;
                 }
               }
#else
               for (i=0; i<CHUNKSIZE; i++) {
                 PServer->rands.entryAt[i] = RAND(PServer->randcount2);
                 PServer->randcount2++;
               }
#endif
             };
             MPI_server_out_chan_$i!rands
          :: else
          fi
";
  }
  print OUT "
       fi;
       if :: !(request>0) -> break :: else fi
     }
  od
}\n\n";
}




# Generate par.prom...

sub generate_par {
  $filename = $_[0];
  $nprocs = $_[1];

  my($i);

  $nworkers = $nprocs-1;

  open(OUT, ">$filename") || die "Could not open $filename";
  print OUT "/*
 * Copyright (c) 2005, University of Massachusetts
 * All Rights Reserved.
 *
 * Name:           monte_par_$nprocs.prom
 * Last modified:  30 Mar 2005
 * Author:         Stephen Siegel <siegel@cs.umass.edu>
 *
 * Parallel version of the Monte Carlo pi estimation program
 * from Using MPI.  This version with $nworkers workers.
 */

";
  generate_mpi;
  foreach $i (0..$nprocs-2) {
    generate_worker($i);
  }
  generate_server;
}




foreach $i (2..10) {
    &generate_par("monte_par_$i.prom", $i);
}
