/*
 * Copyright (c) 2005, University of Massachusetts
 * All Rights Reserved.
 * 
 * Name:           monte_support.c
 * Last modified:  30 Mar 2005
 * Author:         Stephen Siegel <siegel@cs.umass.edu>
 *
 * C support code for the composite model of the Monte Carlo
 * pi estimation program from Using MPI.
 */

#include "../symcode/seqpar.c"

extern void initSymVars();
extern void printUpdate();
extern void printReport();
extern void printGlobalVals(int);


void initializeSystem() {
  int i;

  init_seqpar();
  for (i=0; i<NRANDS+3; i++) {
    SYM_symbolicConstant_0(i);
  }
  SYM_intConstant_0(2);
  SYM_intConstant_0(4);
  printf("Beginning verification of Montel Carlo pi program.\n\n");
  printf("           NPROCS = %d\n", NPROCS);
  printf("        CHUNKSIZE = %d\n", CHUNKSIZE);
  printf("     POINTS_BOUND = %d\n", POINTS_BOUND);
  printf("\n");
  printf("         NWORKERS = %d\n", NWORKERS);
  printf("    CHUNKSIZE_SEQ = %d\n", CHUNKSIZE_SEQ);
  printf("        MAXPOINTS = %d\n", MAXPOINTS);
  printf("           NRANDS = %d\n", NRANDS);
  printf("\n");
  printf("Sequence of random numbers:\n");
  for (i=0; i<NRANDS; i++) {
    printf("  ");
    SYM_printExpr(RAND(i));
    printf("\n");
  }
  printf("\nEPSILON : ");
  SYM_printExpr(EPSILON);
  printf("\nLITERAL_PI : ");
  SYM_printExpr(LITERAL_PI);
  printf("\nINT_MAX : ");
  SYM_printExpr(INT_MAX);
  printf("\n\n");
  printf("-------------------------------");
  printf("--------------------------------------\n\n");

}


void report_par_error(int type, int local, int global) {
  incr_error();
  if (type == 1) {
    printf("  Difference in calculation of totalin:\n\n");
  }
  else if (type == 2) {
    printf("  Difference in calculation of totalout:\n\n");
  }
  else if (type == 3) {
    printf("  Difference in calculation of Pi:\n\n");
  }
  SYM_printExpr(global);
  printf("\n\n");
  SYM_printExpr(local);
  printf("\n");
}



void printSolution(int inoutsum) {
  update_seqpar();
#if VERBOSE>0
  printf("\nin+out = ");
  SYM_printExpr(inoutsum);
  printf("\n\nin = ");
  SYM_printExpr(now.in_global);
  printf("\n\nout = ");
  SYM_printExpr(now.out_global);
  printf("\n\nPi = ");
  SYM_printExpr(now.Pi_global);
  printf("\n\n");
#endif
}
