/*
 * Copyright (c) 2005, University of Massachusetts
 * All Rights Reserved.
 * 
 * Name:           seqpar.c
 * Last modified:  30 Mar 2005
 * Author:         Stephen Siegel <siegel@cs.umass.edu>
 *
 * These functions are used to support the creation of composite
 * models (sequential/parallel) of computational parallel programs.
 */

int numSeqExecs = 0;
int max_pc_len = 0;
int par_errors = 0;
int total_par_errors = 0;

void init_seqpar() {
  SYM_init();
  now.pc = SYM_TRUE;
\#ifndef SYMTYPE
\#define SYMTYPE 0
\#endif
  SYM_setEquivalenceType(SYMTYPE);
}


void printReport() {
  int t = SYM_getEquivalenceType();

  printf("Number of sequential executions: %d\n", numSeqExecs);
  printf("Final table length:              %d\n", SYM_tableLength());
  printf("Max chain length:                %d\n", SYM_maxChainLength());
  printf("Maximum path condition length:   %d\n", max_pc_len);
  printf("Number of errors:                %d\n", total_par_errors);
  printf("Equivalence type:                ");
  if (total_par_errors==0) {
    if (t == SYM_HERBRAND) printf("Herbrand");
    else if (t == SYM_IEEE754) printf("IEEE754");
    else if (t == SYM_REAL) printf("real");
  }
  else {
    printf("none");
  }
  printf("\n\n");
  if (total_par_errors == 0) {
    printf(" **************************************************\n");
    printf(" *   Parallel and sequential models correspond!   *\n");
    printf(" **************************************************\n\n");
  }
  else {
    printf(" ******************************************************\n");
    printf(" * Parallel and sequential models may NOT correspond! *\n");
    printf(" ******************************************************\n\n");
  }
}

void update_seqpar() {
  int pc_len = SYM_pathConditionLength(now.pc);

  if (pc_len > max_pc_len) {
    max_pc_len = pc_len;
  }
#if VERBOSE>0
  printf("Solution %d:\n\n", numSeqExecs);
  SYM_printPathCondition(now.pc);
#endif
  numSeqExecs++;
  par_errors = 0;
}



void printParallelSummary() {
  int t = SYM_getEquivalenceType();

#if VERBOSE>0
  if (par_errors == 0) {
    printf("All parallel solutions are ");
    if (t == SYM_HERBRAND) printf("Herbrand");
    else if (t == SYM_IEEE754) printf("IEEE754");
    else if (t == SYM_REAL) printf("real");
    printf("-equivalent to the sequential solution.\n\n");
  }
  else {
    printf("Possible violation found.\n\n");
  }
  printf("-------------------------------");
  printf("--------------------------------------\n\n");
#endif
}


void incr_error() {
  par_errors++;
  total_par_errors++;
  printf("POSSIBLE ERROR: A parallel solution has been found which does\n");
  printf("NOT match the sequential solution.\n");
}
