/* This code is taken from the CD ROM accompanying the book Parallel
 * Scientific Computing in C++ and MPI, by Karniadakis and Kirby.  It
 * has been slightly modified, primarily to make produce the smaller
 * self-contained version here, and to correct an error in the
 * computation of the error term.  */

#include <iostream.h>
#include <iomanip.h>

class SCMatrix {
private:
  int rows, columns;
  double **data;
public:
  SCMatrix(int rows1, int columns1);
};


SCMatrix::SCMatrix(int rows1, int columns1){
  rows = rows1;
  columns = columns1;

  data = new double*[rows];
  for(int i=0;i<rows;i++){
    data[i] = new double[columns];
    for(int j=0;j<columns;j++)
      data[i][j] = 0.0;
  }
}


double ** CreateMatrix(int m, int n){
  double ** mat;
  mat = new double*[m];
  for(int i=0;i<m;i++){
    mat[i] = new double[n];
    for(int j=0;j<m;j++)
      mat[i][j] = 0.0;
  }
  return mat;
}


void DestroyMatrix(double ** mat, int m, int n){
  for(int i=0;i<m;i++)
    delete[] mat[i];
  delete[] mat;
}


double dot(int N, double *a, double *b){
  double sum = 0.0;
  
  for(int i=0;i<N;i++)
    sum += a[i]*b[i];

  return sum;
}


int Jacobi(int N, double **A, double *x, double *b, double abstol){
  int i,j,k;
  int maxit = 100000;
  double error_sum;
  double sum1,sum2;
  double * xold = new double[N];
  
  //set initial guess to all 1.0
  for(i=0; i<N; i++){
    xold[i] = 1.0;
  }
  
  for(k=0; k<maxit; k++){
    for(i = 0; i<N; i++){
      sum1 = 0.0; sum2 = 0.0;
      for(j=0; j < i; j++)
	sum1 = sum1 + A[i][j]*xold[j];
      for(j=i+1; j < N; j++)
	sum2 = sum2 + A[i][j]*xold[j];
      
      x[i] = (-sum1 - sum2 + b[i])/A[i][i];
    }

    error_sum = 0;
    for (i = 0; i < N; i++) 
      error_sum += (x[i]-xold[i])*(x[i]-xold[i]);
    if(sqrt(error_sum)<abstol){
      delete[] xold;
      return k;
    }
    
    for(i=0; i<N; i++)
      xold[i] = x[i];
  }

  cerr << "Jacobi: Maximum Number of Interations Reached Without Convergence\n";
  delete[] xold;
  
  return maxit;
}



int main(int argc, char * argv[]){
  int i,j, N = 20;
  double **A,*x,*q;
  int answer;

  A = CreateMatrix(N,N);
  x = new double[N];
  q = new double[N];

  for(i=0;i<N;i++){
    q[i] = i+1;
    A[i][i] = -2.0;
    if(i<N-1){
      A[i][i+1] = 1.0;
      A[i+1][i] = 1.0;
    }
  }

  answer = Jacobi(N,A,x,q,1.0e-14);
  cout << "Number of iterations: " << answer << "\n";
  for(i=0;i<N;i++)
    cout << x[i] << endl;
  DestroyMatrix(A,N,N);
  delete[] x;
  delete[] q;
}
