/*
 * Copyright (c) 2004, University of Massachusetts
 * All Rights Reserved.
 * 
 * Name:       matmat_seq.c
 * Date:       08 Jul 2004
 * Revised:    09 Jul 2004
 * Author:     Stephen Siegel <siegel@cs.umass.edu>
 * Maintainer: Stephen Siegel
 * Reader:
 *
 * Compile:    cc matmat_seq.c
 * Run:        ./a.out
 *
 * This is a sequential matrix multiplication program, to serve
 * as a specification for the parallel one.
 */

#include<stdio.h>

#define MAX_AROWS 20
#define MAX_ACOLS 100
#define MAX_BCOLS 20
#define MAX_A_ENTRIES 2000
#define MAX_B_ENTRIES 2000
#define MAX_C_ENTRIES 400

int main(int argc,char *argv[]) {

  double a[MAX_A_ENTRIES], b[MAX_B_ENTRIES], c[MAX_C_ENTRIES];
  double buffer[MAX_ACOLS], ans[MAX_ACOLS];
  double tmp;
  int i, j, k;
  int arows, acols, brows, bcols, crows, ccols;

  arows  = 20;
  acols  = 4;
  brows  = 4;
  bcols  = 3;
  crows  = arows;
  ccols  = bcols;

  for (i = 0; i < acols; i++) {
    for (j = 0; j < arows; j++) {
      a[j*acols+i] = i+1;
    }
  }
  for (i = 0; i < bcols; i++) {
    for (j = 0; j < brows; j++) {
      b[j*bcols+i] = i+1;
    }
  }
  for (i = 0; i < arows; i++) {
    for (j = 0; j < acols; j++) {
      printf("a[%d,%d]=%f\n", i, j, a[i*acols+j]);
    }
  }
  printf("\n");
  for (i = 0; i < brows; i++) {
    for (j = 0; j < bcols; j++) {
      printf("b[%d,%d]=%f\n", i, j, b[i*bcols+j]);
    }
  }
  printf("\n");
  for (i = 0; i < crows; i++) {
    for (j = 0; j < ccols; j++) {
      tmp = 0;
      for (k = 0; k < acols; k++) {
	tmp += a[i*acols+k]*b[k*bcols+j];
      }
      c[i*ccols+j] = tmp;
      printf("c[%d,%d]=%f\n", i, j, c[i*ccols+j]);
    }
  }
}
