/*
 * Copyright (c) 2005, University of Massachusetts
 * All Rights Reserved.
 * 
 * Name:           monte_seq.c 
 * Last modified:  30 Mar 2005
 * Author:         Stephen Siegel <siegel@cs.umass.edu>
 *
 * This is a sequential version of the Monte Carlo Pi estimation
 * program that appears in Using MPI, by Gropp, Lusk, and Skjellum.
 */

#include <math.h> 

/* NOTE: to compare this with the parallel version, the CHUNKSIZE here
 * should be (NPROCS-1)*CHUNKSIZE_par. */
#define CHUNKSIZE_SEQ      4000 
#define INT_MAX 1000000000 
 
int main( int argc, char *argv[] ) { 
  int done, totalin, totalout, rand;
  double x, y, Pi, error, epsilon;
 
  sscanf( argv[1], "%lf", &epsilon ); 
  done = totalin = totalout = 0;
  while (!done) {
    do rand = random(); while (rand > INT_MAX);
    x = (((double) rand)/INT_MAX) * 2 - 1;
    do rand = random(); while (rand > INT_MAX);
    y = (((double) rand)/INT_MAX) * 2 - 1;
    if (x*x + y*y < 1.0) totalin++; else totalout++;
    if ((2*(totalin+totalout))%CHUNKSIZE_SEQ==0) {
      Pi = (4.0*totalin)/(totalin + totalout); 
      error = fabs( Pi-3.141592653589793238462643); 
      done = (error < epsilon || (totalin+totalout) > 1000000);
      printf( "\rpi = %23.20f", Pi );
    }
  }
  printf( "\npoints: %d\nin: %d, out: %d, <ret> to exit\n", 
	  totalin+totalout, totalin, totalout ); 
  getchar();
}
